# =============================================================================
# C O P Y R I G H T
# -----------------------------------------------------------------------------
# Copyright (c) 2020-2022 by Helmut Konrad Fahrendholz. All rights reserved.
# This file is property of Helmut Konrad Fahrendholz. Any unauthorized copy,
# use or distribution is an offensive act against international law and may
# be prosecuted under federal law. Its content is company confidential.
# =============================================================================

import os

DEFAULT_URL = 'http://www.checkitweg.de/writing/'


def docs_url() -> str:
    """Public writers club path.

    >>> docs_url()
    'http://.../writing/'
    """
    url = os.environ.get(
        'CAELUM_DOCS_MAIN',
        DEFAULT_URL,
    )
    assert url.endswith('/'), str(url)
    return url
