# =============================================================================
# C O P Y R I G H T
# -----------------------------------------------------------------------------
# Copyright (c) 2021-2022 by Helmut Konrad Fahrendholz. All rights reserved.
# This file is property of Helmut Konrad Fahrendholz. Any unauthorized copy,
# use or distribution is an offensive act against international law and may
# be prosecuted under federal law. Its content is company confidential.
# =============================================================================

import utilo

import configos.cli.plan
import tests.hvexample


def test_plan_create():
    """Ensure that hvexample is loaded to ensure that hv is loaded correctly."""
    path = utilo.path_parent(tests.hvexample.__file__)
    todo = [path]
    plan = configos.cli.plan.create(todo)
    assert 'TESTS.HVEXAMPLE.SECOND' in plan
